import requests

class IS_handler(object):

    def __init__(self):
        self.enabled = True
        self.name = 'The Archive.is'
        self.api_required = False

    def push(self, uri_org):
        msg = ''
        try:

            archiveTodaySubmitId = ""
            archiveTodayUserAgent = { "User-Agent": "Mozilla/5.0 (X11; Linux x86_64)" }

            # get the newest submitid required to push pages
            rid = requests.get('http://archive.is/', timeout=120, allow_redirects=True, headers=archiveTodayUserAgent)
            rid.raise_for_status()
            htmldata = str(rid.content)
            
            try:
                archiveTodaySubmitId = htmldata.split('name="submitid',1)[1].split('value="',1)[1].split('"',1)[0]
            except: 
                msg = "IndexError (" + self.name+ "): unable to extract 'submitid' "
                raise  

            # push to the archive
            r = requests.post('http://archive.is/submit/', timeout=120,
                                                           data={"anyway":"1" , "url":uri_org, "submitid":archiveTodaySubmitId},
                                                           allow_redirects=True,
                                                           headers=archiveTodayUserAgent)          
            r.raise_for_status()
            # extract the link to the archived copy
            if 'Refresh' in r.headers:
                try:
                    return str(r.headers['Refresh']).split(';url=')[1]
                except:
                   msg = "IndexError (" + self.name+ "): unable to extract a URL to the archived version from HTTP Refresh header ("+str(r.headers['Refresh'])+")"
                   raise 
            else:
                if 'Location' in r.headers:
                    return r.headers['Location']
                else:
                    for r2 in r.history:
                        if 'Location' in r2.headers:
                            return r2.headers['Location']
            msg = "Error ("+self.name+ "): No HTTP Location/Refresh header is returned in the response" 
        except Exception as e:
            if msg == '':
                msg = "Error (" + self.name+ "): " + str(e)
            pass;
        return msg



import requests

class IA_handler(object):

    def __init__(self):
        self.enabled = True
        self.name = 'The Internet Archive'
        self.api_required = False

    def push(self, uri_org):
        msg = ''
        try:
            uri = 'https://web.archive.org/save/' + uri_org
            # push into the archive
            r = requests.get(uri, timeout=120, allow_redirects=True)
            r.raise_for_status()
            # extract the link to the archived copy
            if (r != None):
                if "Location" in r.headers:
                    return r.headers["Location"]
                elif "Content-Location" in r.headers:
                    return "https://web.archive.org"+r.headers["Content-Location"]    
                else:
                    for r2 in r.history:
                        if 'Location' in r2.headers:
                            return r2.headers['Location']
                        if 'Content-Location' in r2.headers:
                            return r2.headers['Content-Location']
            msg = "("+self.name+ "): No HTTP Location/Content-Location header is returned in the response"               
        except Exception as e:
            if msg == '':
                msg = "Error (" + self.name+ "): " + str(e)
            pass;      
        return msg



#!/usr/bin/env python
import os
import re
import sys
import glob
import json
import importlib
import argparse	
from flask import request, Flask, jsonify
from __init__ import __version__ as archiveNowVersion

# archive handlers path
PATH = str(os.path.dirname(os.path.abspath(__file__)))
PATH_HANDLER = PATH + '/handlers/'

# for the web app
app = Flask(__name__)

# create handlers for enabled archives
global handlers
handlers = {}

# defult value for server/port
SERVER_IP = '127.0.0.1'
SERVER_PORT = 12345


def bad_request(error=None):
    message = {
        'status': 400,
        'message': 'Error in processing the request',
    }
    resp = jsonify(message)
    resp.status_code = 400
    return resp

def getServer_IP_PORT():
    u = str(SERVER_IP)
    if str(SERVER_PORT) != '80':
        u = u + ":"+str(SERVER_PORT)
    if 'http' != u[0:4]:
        u = 'http://' + u
    return u


def listArchives_server(handlers):

    li = {"archives": [{"id":"all","GET":getServer_IP_PORT()+'/all/'+'{URI}',"archive-name":"All enabled archives"}] }
    for handler in handlers:
        li["archives"].append({"id":handler,"archive-name":handlers[handler].name,
            "GET":getServer_IP_PORT()+'/'+handler+'/'+'{URI}'})
    return li


@app.route('/', defaults={'path': ''}, methods=['GET'])
@app.route('/<path:path>', methods=['GET'])
def pushit(path):
    # no path; return a list of avaliable archives
    if  path == '':
        resp = jsonify(listArchives_server(handlers))
        resp.status_code = 200
        return resp
    # get request with path 
    else:
        try:
            # get the args passed to push function like API KEY if provided
            PUSH_ARGS = {}
            for k in request.args.keys():
                PUSH_ARGS[k] = request.args[k]

            s = str(path).split('/',1)  
            arc_id = s[0]
            URI = s[1] 

            # To push into archives
            resp = {"results":push(URI, arc_id, PUSH_ARGS)}
            if len(resp["results"]) == 0:
                return bad_request();
            else:
                # what to return
                resp = jsonify(resp)
                resp.status_code = 200
                #resp.headers['Location'] = 
                return resp;
        except Exception as e:
            pass;
        return bad_request();    

def push(URI,arc_id, p_args={}):
    global handlers
    try:
        # push to all possible archives
        res = []
        if arc_id == 'all':
            for handler in handlers:
                if (handlers[handler].api_required):
                    # pass args like key API
                    res.append(handlers[handler].push(str(URI),p_args))
                else:    
                    res.append(handlers[handler].push(str(URI)))
        else:
            # push to the chosen archives 
            for handler in handlers:
                if (arc_id == handler) and (handlers[handler].api_required):
                    res.append(handlers[handler].push(str(URI), p_args))
                elif (arc_id == handler):
                    res.append(handlers[handler].push(str(URI)))    
        return res
    except:
        #print (e)
        pass;
    return ["bad request"]

def start(port=SERVER_PORT, host=SERVER_IP):
    global SERVER_PORT
    global SERVER_IP
    SERVER_PORT = port
    SERVER_IP = host
    app.run(host=host, port=port, threaded=True, debug=True, use_reloader=False)  


def load_handlers():
    global handlers
    handlers = {}
    # add the path of the handlers to the system so they can be imported
    sys.path.append(PATH_HANDLER)

    # create a list of handlers. 
    for file in glob.glob(PATH_HANDLER+'/'+'*.py'):
        sfile = file.rsplit('_',1)
        if len(sfile) == 2:
            
            strRight = sfile[1]
            strLeft =  sfile[0].rsplit("/",1)
            if len(strLeft) > 1:
                strLeft = strLeft[1]
            else:
                strLeft = strLeft[0]   

            if re.match("^[A-Za-z0-9_-]*$", strLeft) and (strRight == 'handler.py'):
                mod = importlib.import_module(strLeft+'_handler') 
                mod_class = getattr(mod, strLeft.upper()+'_handler')
                # finally an object is created
                handlers[strLeft] = mod_class()
    
    # exclude all disabled archives
    for handler in handlers.keys():
        if not handlers[handler].enabled:
            del handlers[handler]


def args_parser():
    global SERVER_PORT 
    global SERVER_IP
    # parsing arguments 
    class MyParser(argparse.ArgumentParser):
        def error(self, message):
            sys.stderr.write('error: %s\n' % message)
            self.print_help()
            sys.exit(2)

        def printm(self):
            sys.stderr.write('')
            self.print_help()
            sys.exit(2)

    parser=MyParser()

    #arc_handler = 0
    for handler in handlers:
        # add archives identifiers to the list of options
        #arc_handler += 1
        parser.add_argument('--'+handler,  action='store_true', default=False, 
   	                        help='Use '+handlers[handler].name)
        if (handlers[handler].api_required):
            parser.add_argument('--'+handler+'_api_key',  nargs='?', 
                            help='An API KEY is required by '+handlers[handler].name)   

    parser.add_argument(
        '-v', '--version', help='Report the version of archivenow', action='version',
        version='ArchiveNow ' + archiveNowVersion)
            
    if len(handlers) > 0:
        parser.add_argument('--all',  action='store_true', default=False, 
                             help='Use all possible archives ')

        parser.add_argument('--server',  action='store_true', default=False, 
                             help='Run archiveNow as a Web Service ')		

        parser.add_argument('URI', nargs='?', help='URI of a web resource')

        parser.add_argument('--host', nargs='?', help='A server address')		

        parser.add_argument('--port', nargs='?', help='A port number to run a Web Service')

        args=parser.parse_args()
    else:
        print ('\n Error: No enabled archive handler found\n')
        sys.exit(0)


    arc_opt = 0;
    # start the server 
    if getattr(args, 'server'):
        if getattr(args, 'port'):
            SERVER_PORT = int(args.port)
        if getattr(args, 'host'):
            SERVER_IP = str(args.host)

        start(port=SERVER_PORT, host=SERVER_IP)

    else: 
        if not getattr(args, 'URI'):
            print (parser.error('too few arguments'))
        res = []
        

        # get the args passed to push function like API KEY if provided
        PUSH_ARGS = {}
        for handler in handlers:
            if (handlers[handler].api_required):
                if getattr(args, handler+'_api_key'):
                    PUSH_ARGS [handler+'_api_key'] = getattr(args, handler+'_api_key')
                else:
                    if getattr(args, handler):
                        print (parser.error('An API KEY is required by '+handlers[handler].name))

        #sys.exit(0)       

        # push to all possible archives   
        if getattr(args, 'all'):
            arc_opt = 1
            res = push(str(args.URI).strip(),'all', PUSH_ARGS)
        else:
            # push to the chosen archives 
            for handler in handlers:
                if getattr(args, handler):
                    arc_opt += 1
                    for i in push(str(args.URI).strip(),handler, PUSH_ARGS):
                        res.append(i)
            # push to the defult archive            
            if (len(handlers) > 0) and (arc_opt == 0):
                # set the default; it ia by default or the first archive in the list if not found
                if 'ia' in handlers:
                    res = push(str(args.URI).strip(),'ia', PUSH_ARGS)
                else:
                    res = push(str(args.URI).strip(),handlers.keys()[0], PUSH_ARGS)  
                #print (parser.printm())
            #else:
        for rs in res:
            print (rs)

load_handlers();

if __name__ == '__main__':
    args_parser();





import random
import string
import requests


class WC_handler(object):

    def __init__(self):
        self.enabled = True
        self.name = 'The WebCite Archive'
        self.api_required = False

    # generate an email required to push pages
    def generate_random_email(self):
        ''' From http://codereview.stackexchange.com/questions/
            58269/generating-random-email-addresses '''
        nb = 1
        length = 7
        domains = [ "hotmail.com", "gmail.com", "aol.com", "mail.com" , "mail.kz", "yahoo.com"]
        letters = string.ascii_lowercase[:12]
        def get_random_domain(domains):
            return random.choice(domains)
        def get_random_name(letters, length):
            return ''.join(random.choice(letters) for i in range(length))
        return [get_random_name(letters, length) + '@' + get_random_domain(domains) for i in range(nb)][0]

    def page_exists_in_wc(self,urim):
        try:
            r = requests.get(urim, timeout=180, 
                            allow_redirects=True)                  

            r2 = requests.get('http://www.webcitation.org/mainframe.php', timeout=180, cookies=r.cookies,
                            allow_redirects=True)  

            if ("When WebCite tried to archive the page, it received a Page Not Found error" in r2.text):   
                return False
            else:
                return True    
        except:
            pass;
        return False        


    def push(self, uri_org):
        msg = ''
        try:
            # push to the archive
            r = requests.post('http://www.webcitation.org/archive', timeout=180, 
                                                                    data={'url':uri_org, 'email':self.generate_random_email()}, 
                                                                    allow_redirects=True)
            r.raise_for_status()
            # extract the link to the archived copy
            if (r != None):
                out = r.text
                try: 
                    urim = 'http://www.webcitation.org/' + out.split('http://www.webcitation.org/')[1][0:9] 
                except:
                   msg = "IndexError (" + self.name+ "): unable to extract a URL to the archived version from the response "
                   raise  
            # check if the page is archived 
            if not self.page_exists_in_wc(urim):
                msg = "Error (" + self.name+ "): " + " Received a Page Not Found error from the website concerned"
            else:
                return str(urim)                                            
        except Exception as e:
            if msg == '':
                msg = "Error (" + self.name+ "): " + str(e)
            pass;
        return str(msg)



import requests
import json

class CC_handler(object):

    def __init__(self):
        self.enabled = True
        self.name = 'The Perma.cc Archive'
        self.api_required = True

    def push(self, uri_org, p_args):
        msg = ''
        try:

            APIKEY = p_args['cc_api_key']

            r = requests.post('https://api.perma.cc/v1/archives/?api_key='+APIKEY, timeout=120,
                                                           data=json.dumps({"url":uri_org}),
                                                           allow_redirects=True)       
            r.raise_for_status()

            if 'Location' in r.headers:
                return 'https://perma.cc/'+r.headers['Location'].rsplit('/',1)[1]
            else:
                for r2 in r.history:
                    if 'Location' in r2.headers:
                        return 'https://perma.cc/'+r2.headers['Location'].rsplit('/',1)[1]
            msg = "Error ("+self.name+ "): No HTTP Location header is returned in the response" 
        except Exception as e:
            if (msg == '') and ('_api_key' in str(e)):
                msg = "Error (" + self.name+ "): " + 'An API KEY is required '
            elif (msg == ''):
                msg = "Error (" + self.name+ "): " + str(e)
            pass;
        return msg



#!/usr/bin/env python

from setuptools import setup, find_packages
from archivenow import __version__

long_description = open('README.rst').read()
desc = """A Python library to push web resources into public web archives"""


setup(
    name='archivenow',
    version=__version__,
    description=desc,
    long_description=long_description,
    author='Mohamed Aturban',
    author_email='maturban@cs.odu.edu',
    url='https://github.com/maturban/archivenow',
    packages=find_packages(),
    license="MIT",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',        
        'License :: OSI Approved :: MIT License'
    ],
    install_requires=[
        'flask',
        'requests'
    ],
    package_data={
        'archivenow': [
            'handlers/*.*'
          ]
    },
    entry_points='''
        [console_scripts]
        archivenow=archivenow.archivenow:args_parser
    '''   
)
